<?php
/* TOP_COMMENT_START
 * Copyright (C) 2022, Levion Partners dba PulseCMS - All Rights Reserved
 *
 * This file is part of Pulse Core. It may be used by individuals or organizations generating less than $400,000 USD per year in revenue, free-of-charge. Individuals or organizations generating over $400,000 in annual revenue who continue to use Pulse Core after 90 days for non-evaluation and non-development use must purchase a paid license. 
 *
 * Proprietary
 * You may modify this source code for internal use. Resale or redistribution is prohibited.
 *
 * You can get the latest version at: https://pulsecms.com/
 *
 * Dated June 2023
 *
TOP_COMMENT_END */

namespace pulsecore\wedge\blog\storage;

require_once (PULSE_BASE_DIR . '/pulsecore/bootstrap.php');

/**
 * build a blog url for linking - using ID and title
 * \param $prefix string prepend this to the result
 * \param $ID string Blog ID
 * \param $title string The blog title
 * \return string
 */
function build_blog_url( $prefix, $id, $title ) {
	static $tempCount = 0;
	$tempCount++;

	\pulsecore\pre_condition(      isset($prefix) );
	\pulsecore\pre_condition( \is_string($prefix) );
	\pulsecore\pre_condition(    \strlen($prefix) >= 0);
	
	\pulsecore\pre_condition(      isset($id) );
	\pulsecore\pre_condition( \is_string($id) );
	\pulsecore\pre_condition(    \strlen($id) > 0);
	
	\pulsecore\pre_condition(      isset($title) );
	\pulsecore\pre_condition( \is_string($title) );
	\pulsecore\pre_condition(    \strlen($title) > 0);
	
	$filtered = \pulsecore\filter\blog_title_in_url($title);

	// EK: added the below code to fix the blog url bug of having multiple blogs
	$pathArray = \explode('/', $prefix);
	// if the last word is not blog, then we do not need to include the blog subdirectory
    if ( strtolower(end($pathArray)) !== "blog") {
		$fullURLPath = \substr($prefix, 0, strpos($prefix, end($pathArray)));
    } else {
		$fullURLPath = $prefix;
	}

	// if there's a / at the end remove it
	if(\substr_compare($fullURLPath, '/', -strlen( '/' )) === 0) {
		$fullURLPath = \substr($fullURLPath, 0, strlen($fullURLPath)-1);
	}

	$result = "{$fullURLPath}/{$filtered}/{$id}";
	//$result = "{$prefix}/{$filtered}/{$id}";
	
	return $result;
}

/**
 * load a blog file
 * \param $filename string
 * \param $location string The blog file location
 * \return stdClass object
 */
function load_blog_file( $filename, $location = '' ) {
	
	\pulsecore\pre_condition(      isset($filename) );
	\pulsecore\pre_condition( \is_string($filename) );
	\pulsecore\pre_condition(    \strlen($filename) > 0);
	\pulsecore\pre_condition(   \is_file($filename) );
	
	\pulsecore\pre_condition(      isset($location) );
	\pulsecore\pre_condition( \is_string($location) );
	\pulsecore\pre_condition(    \strlen($location) >= 0);
			
	$content = \file_get_contents($filename);
	
	$result = parse($content, $filename);
	
	return $result;
}

/**
 * factory for creating the blog storage object
 * \param $description string
 * \param $tags string Comma separated tags
 * \param $location string The blog file location
 * \return stdClass object
 */
function make_store( $description, $tags, $location = '' ) {
	
	$result               = new \stdClass();
	$result->date         = \date('Y-m-d');
	$result->description  = '';
	$result->html         = '';
	$result->id           = '';
	$result->location     = $location;
	$result->relative_url = '';
	$result->tags         = array();
	$result->title        = '';
	
	#description
	if (\strlen($description) > 0) {
		$result->description = \trim($description);
	}
	
	#tags
	if (\strlen($tags) > 0) {
		$cleaned = \trim($tags);
		
		$tmp = \explode(',', $cleaned);
		
		foreach ($tmp as $value) {
			
			$value = \trim($value);
			
			if (\strlen($value) > 0) {
				$result->tags[] = $value;
			}
		}
	}
	
	return $result;
}

/**
 * parse the blog storage file data
 * extract the new blog data
 * \param $content string (sometimes is false)
 * \param $location string The blog file location
 * \return array of the cleaned data and the extracted new data
 */
function parse( $content, $location = '' ) {
	
	$ddd = make_store( '', '', $location );
	
	list($cleaned, $result) = \pulsecore\store\Base::extract( $content, $ddd );
	
	#extract rest of the data
	$data = \explode( "\n", $cleaned );
	
	$result->date  = isset($data[2]) ? $data[2] : '';
	$result->html  = \implode( "\n", \array_slice($data, 3) );
	$result->id    = \basename( $location, '.txt' );
	$result->title = isset($data[0]) ? $data[0] : '';
	
	$result->date  = \trim($result->date );
	$result->html  = \trim($result->html );
	$result->title = \trim($result->title);
	
	$result->relative_url = 'blog-' . \str_replace('.txt', '', $result->location) . '-' . \str_replace(" ", "-", $result->title);
	
	return array($cleaned, $result);
}

/**
 * parse blog url
 * \param $url string
 * \return stdClass unpacked url data
 */
function parse_blog_url( $url ) {
	
	\pulsecore\pre_condition(      isset($url) );
	\pulsecore\pre_condition( \is_string($url) );
	\pulsecore\pre_condition(    \strlen($url) > 0);
	
	$result = (object)array(
		'is_blog_url' => false,
		'blog_id'     => ''
	);
	
	$url_info = $_SERVER['REQUEST_URI'];
	
	$url_broken = $url_info;
	
	$url_broken = \rtrim($url_broken, '/' );
	$url_broken = \explode('/', $url_broken);
	$url_broken = \array_pop( $url_broken );
	
	if ((\stripos($url_broken, 'blog') === 0) and ((\stripos($url_broken, 'blog-page') !== 0)) and (\stripos($url_broken, '-') > 0)) {
		
		$result->is_blog_url = true;
		
		# blog id
		$blog_id = \explode('-', $url_broken);
		$blog_id = $blog_id[1];
		
		$result->blog_id = $blog_id;
	}
	
	return $result;
}

/**
 * Convert a tag list to html
 * \deprecated
 * \param $expanded stdClass The data
 * \param $base_url string The base url path
 * \return string
 */
function tags_to_html( \stdClass $expanded, $base_url ) {
	
	\pulsecore\pre_condition(      isset($base_url) );
	\pulsecore\pre_condition( \is_string($base_url) );
	\pulsecore\pre_condition(    \strlen($base_url) >= 0);
	
	$result = array();
	
	foreach ($expanded->tags as $value) {
		
		$encoded_value = \urlencode($value);
		
		$tmp = "<li><a href=\"{$base_url}/tagged/{$encoded_value}\">#{$value}</a></li>";
		
		$result[] = $tmp;
	}
	
	$result = \implode("\n", $result);
	
	$result  = '<ul class="pulsecore_blog_tag_list">' . $result . '</ul>';
	$result .= "\n";
	$result .= '<div class="pulsecore_float_barrier"></div>';
	
	return $result;
}
